-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

function BeneCastButton:Resize(buttonsize)

	local button, buttonborder, buttoncooldown, buttonfade, buttonrange
	-- Set Button height and width to buttonsize
	self:SetHeight(buttonsize);
	self:SetWidth(buttonsize);
	buttonborder = self.NormalTexture;
	-- Set the Button border to fit around the button
	buttonborder:SetHeight(buttonsize * 2 - (buttonsize/36) * 10);
	buttonborder:SetWidth(buttonsize * 2 - (buttonsize/36) * 10);
	buttoncooldown = self.Cooldown;
	-- Set Cooldown size
	buttoncooldown:SetScale((buttonsize / 36));
	buttoncooldown:SetPoint('CENTER', button, 'CENTER');
	buttonfade = self.Fade;
	-- Set Fade size
	buttonfade:SetHeight(buttonsize * 2 - (buttonsize/36) * 10);
	buttonfade:SetWidth(buttonsize * 2 - (buttonsize/36) * 10);
	buttonrange = self.Range;
	-- Set Range size
	buttonrange:SetHeight(buttonsize);
	buttonrange:SetWidth(buttonsize);

end

function BeneCastButton:GetButtonFromOld()

	local button;
	local len = table.getn(wv.OldButtons);
	if len > 0 then
		button = wv.OldButtons[len];
		table.remove(wv.OldButtons);
	end
	return button;
	
end

-- Make a tooltip for the BeneCastButton based on the options set
function BeneCastButton:OnEnter()

	local profileid = BeneCast:GetCurrentProfileID();
	
	if self.action then
		local tooltiptype, spellbookid, bag, slot, name;
		if (self.action.origin == 'spell' and self.action.spellbookid) or
		   (self.action.origin == 'item' and self.action.bag and self.action.slot) then
			tooltiptype = self.action.origin;
		else
			if (self.action.attribs.type == 'spell' and self.action.spellbookid) or
			   (self.action.attribs.type == 'item' and self.action.bag and self.action.slot) then
				tooltiptype = self.action.attribs.type;
			else
				tooltiptype = 'other';
			end
		end
		name = self.action.name;
		
		if tooltiptype == 'item' then
			local ItemLink = GetContainerItemLink(self.action.bag, self.action.slot);
			local found, _, itemString = string.find(ItemLink, "^|c%x+|H(.+)|h%[.*%]");
			if not found or itemString ~= name then
				tooltiptype = 'other';
			end
		end
		
		if tooltiptype == 'spell' then
			spellbookid = self.action.spellbookid;
			if ( wv.Profiles[profileid].showtooltips and not wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetSpell(spellbookid, SpellBookFrame.bookType);
				GameTooltip:Show();
			elseif ( wv.Profiles[profileid].showtooltips and wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetText(name);
				GameTooltip:Show();
			end
		end
		if tooltiptype == 'item' then
			bag = self.action.bag;
			slot = self.action.slot;
			if ( wv.Profiles[profileid].showtooltips and not wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetBagItem(bag, slot);
				GameTooltip:Show();
			elseif ( wv.Profiles[profileid].showtooltips and wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetText(name);
				GameTooltip:Show();
			end
		end
		if tooltiptype == 'other' then
			if wv.Profiles[profileid].showtooltips then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetText(name);
				GameTooltip:Show();
			end
		end
	end
	if self.reason and self.reason ~= '' then
		BeneCast:Debug('Fade-reason: ' .. self.reason);
	end

end

function BeneCastButton:OnLeave()
	GameTooltip:Hide();
end

function BeneCastButton:PreClick()

end

function BeneCastButton:PostClick()

end

function BeneCastButton:SPELL_UPDATE_COOLDOWN()
	self:UpdateCooldown();
end

function BeneCastButton:OnShow()
	self:UpdateCooldown();
end

-- Update the cooldown for a button on events
function BeneCastButton:UpdateCooldown()

	-- If the unit for this button does not exist return
	if not UnitExists(self.unit) then
		return;
	end
	
	-- No need to update for a hidden button
	if ( not self:IsVisible() ) then
		return;
	end
	
	if self.action then
		if self.action.origin == 'spell' then
			-- Funky Cooldown stuff that I'm not quite sure how exactly it all works
			local start, duration, enable = GetSpellCooldown(self.action.name);
			CooldownFrame_SetTimer(self.Cooldown, start, duration, enable);
			if ( enable == 1 ) then
				self.Icon:SetVertexColor(1.0, 1.0, 1.0);
			else
				self.Icon:SetVertexColor(0.4, 0.4, 0.4);
			end
		end
		if self.action.origin == 'item' then
			local start, duration, enable = GetSpellCooldown(self.action.name);
			CooldownFrame_SetTimer(self.Cooldown, start, duration, enable);
			if ( enable == 1 ) then
				self.Icon:SetVertexColor(1.0, 1.0, 1.0);
			else
				self.Icon:SetVertexColor(0.4, 0.4, 0.4);
			end
		end
	end

end

-- Register events for BeneCastButtons
function BeneCastButton:OnLoad()

	self:RegisterEvent('SPELL_UPDATE_COOLDOWN');

end

function BeneCastButton:SetFade(showbutton, reason)
	self.reason = reason;
	if showbutton then
		self.Fade:Hide();
	else
		self.Fade:Show();
	end
end

function BeneCastButton:NewButton(panel, prevbutton)

	if InCombatLockdown() then
		return;
	end
	local seqno = prevbutton.seqno;
	if seqno == nil then
		seqno = 1;
	else
		seqno = seqno + 1;
	end
	local buttonname = 'BeneCastButton_' .. pfid .. '_' .. unit .. '_' .. seqno;
	local button = self:GetButtonFromOld();
	if not button then
		button = CreateFrame('Button', buttonname , panel, 'BeneCastButtonTemplate');
		button.NormalTexture = getglobal(buttonname .. 'NormalTexture');
		button.Cooldown = getglobal(buttonname .. 'Cooldown');
		button.Fade = getglobal(buttonname .. 'Fade');
		button.Range = getglobal(buttonname .. 'Range');
		button.Duration = getglobal(buttonname .. 'Duration');
		hp._CopyTableToExisting(BeneCastButton,button,true);
		button:SetFrameStrata('LOW');
		button:SetScript('PostClick',self.PostClick);
		button:SetScript('PreClick',self.PreClick);
		button:SetScript('OnEnter',self.OnEnter);
		button:SetScript('OnEvent',function(event, ...)
			if self[event] then
				self[event](...);
			end
		end);
		button:SetScript('OnShow',self.OnShow);
		button:SetScript('OnLeave',self.OnLeave);
		button:SetScript('OnLoad',self.OnLoad);
	else
		button:SetName(buttonname);
		button.NormalTexture:SetName(buttonname .. 'NormalTexture');
		button.Cooldown:SetName(buttonname .. 'Cooldown');
		button.Fade:SetName(buttonname .. 'Fade');
		button.Range:SetName(buttonname .. 'Range');
		button.Duration:SetName(buttonname .. 'Duration');
		button:SetParent(panel);
		button:SetFrameLevel(panel:GetFrameLevel() + 1);
	end
	button:OnLoad();
	button.pf = panel.pf;
	button.unit = panel.unit;
	button.seqno = seqno;
	button.panel = panel;
	button.previous = prevbutton;
	button:SetAttribute('unit',button.unit);
	button:Hide();
	table.insert(panel.buttons,button);
	if prevbutton.next then
		button.next = prevbutton.next;
		prevbutton.next = button;
		panel:ScanButtons();
	else
		prevbutton.next = button;
	end
		
	return button;
end

-- Function to hide buttons if they're not to be shown
function BeneCastButton:Retire()

	if InCombatLockdown() then
		return;
	end

	-- First hide his members
	self.Fade:Hide();
	self.Range:Hide();
	self:Hide();
	
	-- Strip him of properties
	self:ClearAction();
	self.unit = nil;
	self.pf = nil;
	self.seqno = nil;
	
	-- Get him out of the panel-buttonchain
	local previous, next = self.previous, self.next;
	previous.next = next;
	if next then
		next.previous = previous;
	end
	self.next = nil;
	self.previous = nil;
	
	-- Make the panel renumber his buttons
	if self.panel then
		self.panel:ScanButtons();
		self.panel:OrientButtons();
		self.panel = nil;
	end
	
	-- Detach him from everything
	button:ClearAllPoints();
	button:SetParent(UIParent);
	button:UnRegisterAllEvents();
	
	-- So long old friend, *snif... :p
	tinsert(wv.OldButtons, button);

end

function BeneCastButton:ClearAction()

	if self.action then
		if self.action.attribs then
			for k, v in pairs(self.action.attribs) do
				self:SetAttribute(k, nil);
			end
		end
		self.action = nil;
	end

end

function BeneCastButton:SetAction(action)

	self:ClearAction();
	self.action = action;
	if self.action then
		if self.action.attribs then
			for k, v in pairs(self.action.attribs) do
				self:SetAttribute(k, v);
			end
		end
		self.Icon:SetTexture(self.action.icon);
	end

end